const channelsID = require('./channels_conf.json');
const complotDB = require('../database/ComplotDB.js');

const CATEGORY = {
    ACTION : 0,
    REASON : 1,
    ACTOR : 2,
    SOURCE : 3,
    SENTENCE : 4
};

class VoteParser{
    
    constructor(client){
        this.client = client;
        this.upVote = '👍';
        this.score = 3;
        this.categories = new Map([
            [channelsID.actor, CATEGORY.ACTOR],
            [channelsID.reason, CATEGORY.REASON],
            [channelsID.action, CATEGORY.ACTION],
            [channelsID.source, CATEGORY.SOURCE],
            [channelsID.sentence, CATEGORY.SENTENCE]
        ]);

        this.infoMessageActor = null;
        this.infoMessageReason = null;
        this.infoMessageAction = null;
        this.infoMessageSource = null;
        this.infoMessageSentence = null;

        this.channelBase = this.client.channels.cache.get(channelsID.base);

    }

    setScore(score) {
        this.score = score;
    }

    parse(message){
        if(this.categories.has(message.channelId)){
            const category = this.categories.get(message.channelId);
            const reactions = message.reactions.resolve(this.upVote);
            if(reactions && reactions.count >= this.score){
                this.add(message, category);
            }
        }
    }

    categoryCase(category, content){
        if(category === CATEGORY.SOURCE || category === CATEGORY.SENTENCE){
            return content;
        }

        const char = content.charAt(0);
        const reste = content.slice(1);
        content = (category === CATEGORY.ACTOR) ? char.toUpperCase() + reste: char.toLowerCase() + reste;
        return content;
    }

    async add(message, category){
        let content = this.categoryCase(category, message.content);
        
        this.addFormatedString(content, category);
        message.delete();
        message.channel.send(`"${content}" ajouté ! \n Par <@${message.author.id}>`);
        
        this.updateBase(category);
    }

    async updateBase(category){
        const actor = complotDB.actorToString();
        const action = complotDB.actionToString();
        const reason = complotDB.reasonToString();
        const source = complotDB.sourceToString();
        const sentence = complotDB.sentenceToString();

        if(!this.infoMessageActor){
            await this.deleteAllMessageInfo();
            this.infoMessageActor = await this.channelBase.send(actor);
            this.infoMessageAction = await this.channelBase.send(action);
            this.infoMessageReason = await this.channelBase.send(reason);
            this.infoMessageSource = await this.channelBase.send(source);
            this.infoMessageSentence = await this.channelBase.send(sentence);
        }else{
            switch(category){
                case CATEGORY.ACTION: this.infoMessageAction.edit(action); break;
                case CATEGORY.REASON: this.infoMessageReason.edit(reason); break;
                case CATEGORY.SOURCE: this.infoMessageSource.edit(source); break;
                case CATEGORY.ACTOR: this.infoMessageActor.edit(actor); break;
                case CATEGORY.SENTENCE: this.infoMessageSentence.edit(sentence);break;
            }
        }
    }




    addFormatedString(messageString, category){
        switch (category){
            case CATEGORY.ACTOR: complotDB.addActor(messageString); break;
            case CATEGORY.ACTION: complotDB.addAction(messageString); break;
            case CATEGORY.REASON: complotDB.addReason(messageString); break;
            case CATEGORY.SOURCE: complotDB.addSource(messageString); break;
            case CATEGORY.SENTENCE: complotDB.addSentence(messageString); break;
        }
    }

    async deleteAllMessageInfo(){

        let messages = await this.channelBase.messages.fetch();
        let workers = [];
        messages.forEach(m=>{
            workers.push(m.delete());
        });
        await Promise.all(workers);
    }

}

module.exports = VoteParser;